@@ -182,6 +182,9 @@ for details.'''
     g_test.add_argument('-B', '--no-built-binaries', dest='built_binaries',
                         action='store_false', default=True,
                         help='do not build/use binaries from .dsc, git source, or unbuilt tree')
+    g_test.add_argument('-a', '--test-architecture', metavar='ARCH',
+                        help='run tests for (and when asked, build binaries '
+                        'for) ARCH instead of the testbed\'s architecture.')
     g_test.add_argument('packages', nargs='*',
                         help='testsrc source package and testbinary packages as above')
 
@@ -205,23 +208,12 @@ for details.'''
     g_setup = parser.add_argument_group('test bed setup options')
     g_setup.add_argument('--setup-commands', metavar='COMMANDS_OR_PATH',
                          action='append', default=[],
-                         help='Run these commands after opening the testbed '
-                         '(e. g. "apt-get update" or adding apt sources); '
+                         help='Run these commands after opening the testbed; '
                          'can be a string with the commands, or a file '
                          'containing the commands')
-    # Ensure that this fails with something other than 100 in most error cases,
-    # as apt-get update failures are usually transient; but if we find a
-    # nonexisting apt source (404 Not Found) we *do* want 100, as otherwise
-    # we'd get eternally looping tests.
-    g_setup.add_argument('-U', '--apt-upgrade', dest='setup_commands',
-                         action='append_const',
-                         const='''(O=$(bash -o pipefail -ec 'apt-get update | tee /proc/self/fd/2') ||'''
-                         '{ [ "${O%404*Not Found*}" = "$O" ] || exit 100; sleep 15; apt-get update; }'
-                         ' || { sleep 60; apt-get update; } || false)'
-                         ' && $(command -v eatmydata || true) apt-get dist-upgrade -y -o '
-                         'Dpkg::Options::="--force-confnew"'
-                         ' && $(command -v eatmydata || true) apt-get --purge autoremove -y',
-                         help='Run apt update/dist-upgrade before the tests')
+    g_setup.add_argument('-U', '--apt-upgrade', dest='apt_upgrade',
+                         action='store_true',
+                         help='Run apt update/dist-upgrade/autoremove before the tests')
     g_setup.add_argument('--setup-commands-boot', metavar='COMMANDS_OR_PATH',
                          action='append', default=[],
                          help='Run these commands after --setup-commands, '
@@ -256,8 +248,7 @@ for details.'''
                          help='Define the default release for apt. For apt pinning to work '
                          'properly, the default release must be set to the release '
                          'that should provide the packages that are not pinned. For '
-                         'Debian and Ubuntu, this is normally automatically detected '
-                         'from /etc/os-release.')
+                         'Debian and Ubuntu, this is normally automatically detected.')
     g_setup.add_argument('--no-apt-fallback',
                          dest='enable_apt_fallback',
                          action='store_false',
@@ -423,4 +414,11 @@ for details.'''
         copy_pairs.append((host, tb))
     args.copy = copy_pairs
 
+    # split multiline add_apt_sources into separate entries
+    args.add_apt_sources = [s for line in args.add_apt_sources for s in line.splitlines()]
+    # strip, drop empty, drop comments, deduplicate
+    args.add_apt_sources = [s.strip() for s in args.add_apt_sources]
+    args.add_apt_sources = [s for s in args.add_apt_sources if s and s[0] != '#']
+    args.add_apt_sources = list(dict.fromkeys(args.add_apt_sources))
+
     return (args, actions, virt_args)
